//! cassiopeia file format

mod ir;
mod lexer;
mod parser;

pub(crate) use ir::{IrItem, IrStream, IrType, MakeIr};
pub(crate) use lexer::{LineLexer, LineToken, Token};
pub(crate) use parser::LineCfg;

use crate::TimeFile;
use std::{fs::File, io::Read};

#[derive(Default)]
pub struct ParseOutput {
    pub(crate) ir: IrStream,
    pub(crate) tf: TimeFile,
}

impl ParseOutput {
    fn append(mut self, ir: IrItem) -> Self {
        self.tf.append(ir.clone());
        self.ir.push(ir);
        self
    }
}

/// Load a file from disk and parse it into a
/// [`TimeFile`](crate::TimeFile)
pub fn load_file(path: &str) -> Option<ParseOutput> {
    let mut f = File::open(path).ok()?;
    let mut content = String::new();
    f.read_to_string(&mut content).ok()?;

    let mut lines: Vec<String> = content.split("\n").map(|l| l.to_owned()).collect();

    Some(
        ir::generate_ir(
            lines
                .iter_mut()
                .map(|line| lexer::lex(line))
                .map(|lex| parser::parse(lex)),
        )
        .into_iter()
        .fold(ParseOutput::default(), |output, ir| output.append(ir)),
    )
}
